% This script generates latex format tables for the publication,
% directly from the matlab processed results.

close all
clear all

load('results.mat')

latex_models = {'\citet{ohara2011}','\citet{tentusscher2006}', '\citet{grandi2010novel}'};
latex_variants = {'IonWorks Quattro','IonWorks Quattro with Barracuda hERG \& CaV1.2', 'IonWorks Quattro with manual hERG'};

results_summary = cell(length(models),length(variants),length(concentration_cut_offs));

for m = 1:length(models)
    for v = 1:length(variants)
        fprintf('%% This table was autogenerated by results_to_latex_tables.m, so don''t edit it here, edit that instead!\n')
        fprintf('\\begin{table}[htbp]\n')
        fprintf('\\begin{center}\n')
        fprintf('\\begin{small}\n')
        fprintf('\\caption{A contingency table for %s predictions based on the %s dataset.\nA match is defined as agreement ',latex_models{m},latex_variants{v})
        fprintf('at estimated TQT concentration (top table), within 10-fold (middle), or within 100-fold (bottom).}\n')

        %for c = 1:length(concentration_cut_offs) % We don't need 1000 fold
        %really...
        for c = 1:(length(concentration_cut_offs)-1)
            %for prolong_or_short = 1:2
            for prolong_or_short = 1 % Only bothered about prolongation for now.

                % Work out totals for each box of a contingency table.
                true_pos = 0;
                false_pos = 0;
                false_neg = 0;
                true_neg = 0;
                % Data stored in this cell as sim_positive, TQT_positive
                for i=1:size(results{m,v,c,prolong_or_short},1)
                    if results{m,v,c,prolong_or_short}(i,1) && results{m,v,c,prolong_or_short}(i,2)
                        true_pos = true_pos +1;
                    end
                    if results{m,v,c,prolong_or_short}(i,1) && ~results{m,v,c,prolong_or_short}(i,2)
                        false_pos = false_pos +1;
                    end
                    if ~results{m,v,c,prolong_or_short}(i,1) && results{m,v,c,prolong_or_short}(i,2)
                        false_neg = false_neg +1;
                    end
                    if ~results{m,v,c,prolong_or_short}(i,1) && ~results{m,v,c,prolong_or_short}(i,2)
                        true_neg = true_neg +1;
                    end
                end

%                 % Sim_Expt names
%                 if (prolong_or_short==1)
%                     fprintf('\nProlongation beyond %g ms:\\\\\n',positive_effect_cut_off)
%                 else
%                     fprintf('\nShortening beyond %g ms:\\\\\n',positive_effect_cut_off)
%                 end
%                 fprintf('Model: %s\\\\\n',latex_models{m})
%                 fprintf('Data: %s\\\\\n',latex_variants{v})
%                 fprintf('Sim correct within %i times TQT conc\\\\\n\n', concentration_cut_offs(c))

                sensitivity = 100.0.*true_pos/(true_pos+false_neg);
                specificity = 100.0.*true_neg/(false_pos+true_neg);
                accuracy = 100.0.*(true_pos+true_neg)./(true_pos+false_pos+false_neg+true_neg);
                ppv = 100.0.*true_pos/(true_pos+false_pos);
                npv = 100.0.*true_neg/(false_neg+true_neg);
                
                results_summary{m,v,c} = [sensitivity, specificity, accuracy, ppv, npv, positive_effect_cut_off];

                fprintf('\\begin{tabular}{llcccl}\n')
                fprintf('\\multicolumn{2}{l}{\\multirow{2}{*}{\\begin{tabular}[l]{@{}l@{}} {%i-fold} \\\\ {conc.}\\end{tabular}}} & \\multicolumn{2}{c}{Thorough QTc change} & \\multicolumn{1}{l}{} &\\\\\n',concentration_cut_offs(c))
                fprintf('\\multicolumn{2}{l}{}&\\multicolumn{1}{|c}{$\\geqslant5$ms}&\\multicolumn{1}{|c}{$<5$ms}&\\multicolumn{1}{|c|}{Totals}& \\\\ \\cline{2-5}\n')
                fprintf('\\multicolumn{1}{c}{\\multirow{2}{*}{\\begin{tabular}[c]{@{}c@{}}Simulated\\\\1Hz\\\\APD\\\\change\\end{tabular}}} & \\multicolumn{1}{l}{$\\geqslant5$ms}&\\multicolumn{1}{|c}{%i}&\\multicolumn{1}{|c}{%i}&\\multicolumn{1}{|c|}{%i}&\\multicolumn{1}{c}{\\begin{tabular}[c]{@{}c@{}}Positive Predictive\\\\Value %3.0f\\%%\\end{tabular} } \\\\ \\cline{2-5}\n',true_pos, false_pos, true_pos+false_pos,ppv)
                fprintf('&\\multicolumn{1}{l}{$<5$ms}&\\multicolumn{1}{|c}{%i} & \\multicolumn{1}{|c}{%i} & \\multicolumn{1}{|c|}{%i} & \\multicolumn{1}{c}{\\begin{tabular}[c]{@{}c@{}}Negative Predictive\\\\ Value %3.0f\\%%\\end{tabular}}  \\\\ \\cline{2-5}\n',false_neg, true_neg, false_neg+true_neg,npv)
                fprintf('&\\multicolumn{1}{l}{Totals}&\\multicolumn{1}{|c}{%i} & \\multicolumn{1}{|c}{%i} & \\multicolumn{1}{|c|}{%i} & \\\\ \\cline{2-5}\n',true_pos+false_neg,false_pos+true_neg,true_pos+false_pos+false_neg+true_neg)
                fprintf('& & \\begin{tabular}[c]{@{}c@{}}Sensitivity %3.0f \\%%\\end{tabular} & \\begin{tabular}[c]{@{}c@{}}Specificity %3.0f\\%%\\end{tabular} & \\multicolumn{1}{l}{}&\\multicolumn{1}{c}{\\begin{tabular}[c]{@{}c@{}}Accuracy %3.0f\\%%\\end{tabular}}\n', sensitivity, specificity, accuracy)
                fprintf('\\end{tabular}\\\\\\vspace{1cm}\n')
                
            end % Prolonger or shortener
        end % conc cut offs                
        fprintf('\\label{tab:ResultsMatrix:%s}\n',[models{m} ':' variants{v}])
        fprintf('\\end{small}\n')
        fprintf('\\end{center}\n')
        fprintf('\\end{table}\n\n')
        
    end % models
end % variants

save('results.mat','results','models','variants','positive_effect_cut_off','concentration_cut_offs','results_summary','-mat')

colours = colormap(autumn(100));

% Print another summary table for O'hara colour coded.
fprintf('%% This table was autogenerated by results_to_latex_tables.m, so don''t edit it here, edit that instead!\n')
fprintf('%% Please remember to add \\use{multirow} to your document preamble in order to suppor multirow cells\n');
fprintf('%% Booktabs require to add \\usepackage{booktabs} to your document preamble.\n')
fprintf('\\begin{table}[htb]\n')
fprintf('\\begin{center}\n')
fprintf('\\caption{A summary of the models'' performance with different datasets and `concentration windows'', when aiming to predict whether 5ms prolongation is observed.\n')
fprintf('Q --- Quattro dataset, M\\&Q --- Quattro with manual hERG (see Table~\\ref{tab:InputData}).}\n')
fprintf('\\vspace{0.2cm}\n')
fprintf('\\begin{tabular}{@{}clccccccccccc@{}}\n')
fprintf('\\toprule\n')
fprintf('\\begin{tabular}[c]{@{}c@{}}Conc.\\\\ range\\end{tabular} & Measure & \\begin{tabular}[c]{@{}c@{}}O''Hara\\\\ Q\\end{tabular} & & \\begin{tabular}[c]{@{}c@{}}O''Hara\\\\ M\\&Q\\end{tabular} & & \\begin{tabular}[c]{@{}c@{}}TT''06\\\\ Q\\end{tabular}& & \\begin{tabular}[c]{@{}c@{}}TT''06\\\\ M\\&Q\\end{tabular} & & \\begin{tabular}[c]{@{}c@{}}Grandi\\\\ Q\\end{tabular}& & \\begin{tabular}[c]{@{}c@{}}Grandi\\\\ M\\&Q\\end{tabular} \\\\ \\midrule\n')
for c=1:(length(concentration_cut_offs)-1)
    fprintf('\\multirow{2}{*}{\\begin{tabular}[c]{@{}c@{}} ')
    if c==1
        fprintf('at')
    else
        fprintf('%i-fold',concentration_cut_offs(c))
    end
    % Sensitivity
    fprintf(' \\\\ TQT conc.\\end{tabular}} & Sensitivity')
    for m=1:3
        hts = results_summary{m,1,c}(1);
        manual = results_summary{m,3,c}(1);
        rgb1 = colours(floor(hts)+1,:);
        rgb2 = colours(floor(manual)+1,:);
        if m>1
            fprintf('& ')
        end
        fprintf('& \\cellcolor[rgb]{%f,%f,%f} %3.0f\\%% & & \\cellcolor[rgb]{%f,%f,%f} %3.0f\\%% ',rgb1(1),rgb1(2),rgb1(3),hts,rgb2(1),rgb2(2),rgb2(3),manual)
    end
    fprintf('\\\\ \\cmidrule(l){2-13} \n')
    % Accuracy
    fprintf('& Accuracy')
    for m=1:3
        hts = results_summary{m,1,c}(3);
        manual = results_summary{m,3,c}(3);
        rgb1 = colours(floor(hts)+1,:);
        rgb2 = colours(floor(manual)+1,:); 
        if m>1
            fprintf('& ')
        end
        fprintf('&\\cellcolor[rgb]{%f,%f,%f} %3.0f\\%% & & \\cellcolor[rgb]{%f,%f,%f} %3.0f\\%%',rgb1(1),rgb1(2),rgb1(3),hts,rgb2(1),rgb2(2),rgb2(3),manual)
    end
    fprintf('\\\\ \\midrule\n')
end
fprintf('\\end{tabular}\n')
fprintf('\\label{tab:ModelSummary}\n')
fprintf('\\end{center}\n')
fprintf('\\end{table}\n')


